/*-----------------------------------------------------------------------
				FedTerm version 1.60 - Windows Federation 2 Client
			  Copyright (c) 1995-2006 Interactive Broadcasting Ltd

	This program is free software: you can redistribute it and /or modify 
	it under the terms of the GNU General Public License as published by 
	the Free Software Foundation: either version 2 of the License, or (at 
	your option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY: without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
	General Public License for more details.

	You should have received a copy of the GNU General Public License 
	along with this program; if not, write to the Free Software Foundation, 
	Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307 USA

	Program written and compiled with Borland C++ Builder 5
	Programming and design: 	Alan Lenton
	Home website:					www.ibgames.net
-----------------------------------------------------------------------*/

#include <vcl.h>
#pragma hdrstop

#include "friends_code.h"

#include <sstream>
#include <cctype>

#include <mmsystem.h>

#include "friends.h"
#include "main.h"
#include <io.h>

//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------

void	Friends::AddFriend(const std::string& name)
{
	std::list<std::string>::iterator	iter;
	for(iter = friends_list.begin();iter != friends_list.end();iter++)
	{
		if(*iter == name)
			return;
	}

	std::string	n_name(Normalise(name));
	friends_list.push_back(n_name);
}

void	Friends::AddRanks(unsigned long flags)
{
	std::bitset<MAX_RANK>	temp_ranks(flags);
	ranks |= temp_ranks;
}

void	Friends::Clear()
{
	friends_list.clear();
	ranks.reset();
}

void	Friends::GetInfo()
{
	Clear();
	for(int count = 0;count < MAX_RANK;count++)
	{
		switch(count)
		{
			case GROUNDHOG:		if(FriendsForm->GHogCheck->Checked)  			ranks.set(count);   break;
			case COMMANDER:		if(FriendsForm->CommanderCheck->Checked)  	ranks.set(count);   break;
			case CAPTAIN:			if(FriendsForm->CaptainCheck->Checked)  		ranks.set(count);   break;
			case ADVENTURER:		if(FriendsForm->AdventCheck->Checked)  		ranks.set(count);   break;
			case MERCHANT:			if(FriendsForm->MerchantCheck->Checked)  		ranks.set(count);   break;
			case TRADER:			if(FriendsForm->TraderCheck->Checked)  		ranks.set(count);   break;
			case INDUSTRIALIST:	if(FriendsForm->IndustrialistCheck->Checked) ranks.set(count);   break;
			case MANUFACTURER:	if(FriendsForm->ManufacturerCheck->Checked)  ranks.set(count);   break;
			case FINANCIER:		if(FriendsForm->FinancierCheck->Checked)  	ranks.set(count);   break;
			case FOUNDER:			if(FriendsForm->FounderCheck->Checked)  		ranks.set(count);   break;
			case ENGINEER:			if(FriendsForm->EngineerCheck->Checked)  		ranks.set(count);   break;
			case MOGUL:				if(FriendsForm->MogulCheck->Checked)  			ranks.set(count);   break;
			case TECHNOCRAT:		if(FriendsForm->TechnoCheck->Checked)  		ranks.set(count);   break;
			case MAGNATE:			if(FriendsForm->MagnateCheck->Checked)  		ranks.set(count);   break;
			case PLUTOCRAT:		if(FriendsForm->PlutocratCheck->Checked)  	ranks.set(count);   break;
			case SYNDICRAT:		if(FriendsForm->SyndicratCheck->Checked)  	ranks.set(count);   break;
		}
	}

	for(int count = 0;count < FriendsForm->FriendsList->Items->Count;count++)
		friends_list.push_back(Normalise(FriendsForm->FriendsList->Items->Strings[count].c_str()));
	audio = FriendsForm->AudioCheck->Checked;
}

std::string&	Friends::Normalise(const std::string& name)
{
	static std::string	n_name;
	n_name = name;
	int len =  n_name.length();
	if(len != 0)
	{
		n_name[0] = std::toupper(n_name[0]);
		for(int count = 1;count < len;count++)
			n_name[count] = tolower(n_name[count]);
	}
	return(n_name);
}

void	Friends::Process(int rank,const std::string& name,bool enter)
{
	std::ostringstream	buffer;
	if(enter && ranks.test(rank))
	{
		if(enter)
		{
			buffer << name << " is logged into the game.";
			MainForm->OutputText(buffer.str());
		}
		SoundAudio();
		return;
	}

	std::list<std::string>::iterator iter;
	for(iter = friends_list.begin();iter != friends_list.end();iter++)
	{
		if(*iter == name)
		{
			if(enter)
				MainForm->AddFriend(name);
			else
				MainForm->DeleteFriend(name);
			buffer << name << " is " << (enter ? "logged into" : "logged out of") << " the game.";
			MainForm->OutputText(buffer.str());
			SoundAudio();
			return;
		}
	}
}

void	Friends::SoundAudio()
{
	if(audio)
	{
		std::string buffer = MainForm->HomeDir() + "\\sounds\\friends.wav";
		if(access(buffer.c_str(),0) != 0)
		{
			buffer = MainForm->HomeDir() + "\\sounds\\alert.wav";
			if(access(buffer.c_str(),0) != 0)
				return;
		}
		sndPlaySound(buffer.c_str(),SND_ASYNC);
	}
}

void	Friends::UpdateForm()
{
	FriendsForm->Clear();
	for(int count = 0;count < MAX_RANK;count++)
	{
		if(ranks.test(count))
			FriendsForm->SetRank(count);
	}
	std::list<std::string>::iterator	iter;
	for(iter = friends_list.begin();iter != friends_list.end();iter++)
		FriendsForm->AddName(*iter);
	if(audio)
		FriendsForm->SetAudio();
}

void	Friends::Write(std::ofstream& file)
{
	if((friends_list.size() > 0) || ranks.any())
	{
		file << "   <friends-list ranks='" << ranks.to_ulong() << "' audio='";
		file << (audio ? "true" : "false") << "'>\n";
		std::list<std::string>::iterator	iter;
		for(iter = friends_list.begin();iter != friends_list.end();iter++)
			file << "      <friend name='" << *iter << "'/>\n";
		file << "   </friends-list>\n\n";
	}
}


