/*-----------------------------------------------------------------------
				FedTerm version 1.60 - Windows Federation 2 Client
			  Copyright (c) 1995-2006 Interactive Broadcasting Ltd

	This program is free software: you can redistribute it and /or modify 
	it under the terms of the GNU General Public License as published by 
	the Free Software Foundation: either version 2 of the License, or (at 
	your option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY: without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
	General Public License for more details.

	You should have received a copy of the GNU General Public License 
	along with this program; if not, write to the Free Software Foundation, 
	Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307 USA

	Program written and compiled with Borland C++ Builder 5
	Programming and design: 	Alan Lenton
	Home website:					www.ibgames.net
-----------------------------------------------------------------------*/

#include <vcl.h>
#pragma hdrstop

#include "init.h"

#include <sstream>

#include "coms.h"
#include "fed2.h"
#include "friends_code.h"
#include "login_code.h"
#include "macro.h"
#include "main.h"
#include "xml.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------


Init::Init(const std::string& f_name)
{
	autologging = false;
	file_name = f_name;
	buffer_size = 200;
	bkg_colour = clWindow;
	voted = 0;
}

Init::~Init()
{
	autologging = false;
	for(AccountsList::iterator iter = accounts_list.begin();iter != accounts_list.end();iter++)
		delete (*iter);
}


void	Init::AddAccount(AccountRec *rec)
{
	accounts_list.push_back(rec);
}

void	Init::AutoLogging(bool is_on)
{
	autologging = is_on;
}

const AccountRec	*Init::DeleteCurrentAccount()
{
	for(AccountsList::iterator iter = accounts_list.begin();iter != accounts_list.end();iter++)
	{
		if((*iter)->current)
		{
			AccountRec	*rec = *iter;
			accounts_list.erase(iter);
			delete rec;
			break;
		}
	}
	Write();
	if(accounts_list.begin() !=accounts_list.end())
		return(*(accounts_list.begin()));
	else
		return(0);
}

void	Init::DumpAccounts()
{
	std::ostringstream	buffer;
	for(AccountsList::iterator iter = accounts_list.begin();iter != accounts_list.end();iter++)
	{
		buffer << "name is " << (*iter)->avatar_name << "\n";
		buffer << "id is " << (*iter)->account_id << "\n";
		buffer << "password is " << (*iter)->password << "\n";
		buffer << "host is " << (*iter)->host << "\n";
		buffer << "port is " << (*iter)->port << "\n";
		MessageDlg(buffer.str().c_str(),mtInformation,TMsgDlgButtons() << mbOK,0);
	}
}

const AccountRec	*Init::GetCurrentAccount()
{
	for(AccountsList::iterator iter = accounts_list.begin();iter != accounts_list.end();iter++)
	{
		if((*iter)->current)
			return(*iter);
	}

	// nothing marked as current - do we have any accounts? If so send the first one
	if(accounts_list.begin() !=accounts_list.end())
	{
		(*(accounts_list.begin()))->current = true;
		return(*(accounts_list.begin()));
	}

	return(0);	// nothing at all - return NULL
}

const AccountRec	*Init::GetNextAccount()
{
	for(AccountsList::iterator iter = accounts_list.begin();iter != accounts_list.end();iter++)
	{
		if((*iter)->current)
		{
			(*iter)->current = false;
			if((++iter) != accounts_list.end())
			{
				(*(iter))->current = true;
				return(*(iter));
			}
			else
				return(0);
		}
	}
	return(0);
}

const AccountRec	*Init::GetPreviousAccount()
{
	for(AccountsList::iterator iter = accounts_list.begin();iter != accounts_list.end();iter++)
	{
		if((*iter)->current)
		{
			if(iter != accounts_list.begin())
			{
				(*iter)->current = false;
				(*(--iter))->current = true;
				return(*(iter));
			}
			else
				return(0);
		}
	}

	// we don't have a current account - try and send the last account
	if(accounts_list.begin() != accounts_list.end())
	{
		AccountsList::iterator iter = accounts_list.end();
		iter--;
		(*iter)->current = true;
		return(*iter);
	}
	return(0);
}

void	Init::NewBkgColour(TColor new_colour)
{
	bkg_colour = new_colour;
	if(bkg_colour != clWindow)
		MainForm->SetColour(bkg_colour);
}

void	Init::UpdateAccount(AccountRec *rec)
{
	AccountRec	*current = 0;
	for(AccountsList::iterator iter = accounts_list.begin();iter != accounts_list.end();iter++)
	{
		if((*iter)->current)
		{
			current = *iter;
			break;
		}
	}

	bool add_account = false;
	if(current == 0)
	{
		current = new AccountRec;
		add_account = true;
	}

	current->host = rec->host;
	current->port = rec->port;
	current->avatar_name = rec->avatar_name;
	current->account_id = rec->account_id;
	current->password = rec->password;
	current->current = rec->current;
	current->save = rec->save;
	if(add_account)
		AddAccount(current);
	Write();
}

void	Init::UpdateColour()
{
	MainForm->SetColour(bkg_colour);
}

void	Init::Write()
{
	std::ofstream	file(file_name.c_str(),std::ios::out | std::ios::trunc);
	if(!file)
	{
		MessageDlg("Unable to save your personal\nsettings to file init.xml",
																mtError,TMsgDlgButtons() << mbOK,0);
		return;
	}

	file << "<?xml version=\"1.0\"?>\n\n";
	file << "<init-file>\n\n";
	file << "   <size width='" << MainForm->Width << "' height='" << MainForm->Height << "'/>\n";
	file << "   <buffer size='" << buffer_size << "'/>\n";
	file << "   <input-window height='" << MainForm->InputPanel->Height << "'/>\n";
	file << "   <colour bkg='" << static_cast<int>(bkg_colour) << "'/>\n";
	file << "   <stats-panel visible='" << (MainForm->StatsItem->Checked ? "true" : "false") << "'/>\n";
	file << "   <audio-alert enabled='" << (MainForm->TextAlertItem->Checked ? "true" : "false") << "'/>\n";
	file << "   <twinkles enabled='" << (MainForm->LightsItem->Checked ? "true" : "false") << "'/>\n";
	file << "   <sound-fx enabled='" << (MainForm->SoundItem->Checked ? "true" : "false") << "'/>\n";

	if(voted > 0)										file << "	<voting time='" << voted << "'/>\n";
	if(MainForm->BlankLinesItem->Checked)		file << "   <no-blank-lines/>\n";
	if(autologging)                      		file << "   <autolog logging='true'/>\n";
	file << "\n";
	
	Fed2::friends->Write(file);
	Fed2::macro->Write(file);
	WriteFont(file);

	for(AccountsList::iterator iter = accounts_list.begin();iter !=  accounts_list.end();iter++)
	{
		file << "   <avatar name='" << (*iter)->avatar_name << "'>\n";
		file << "      <server host='" << (*iter)->host << "' port='" << (*iter)->port << "'/>\n";
		file << "      <account id='" << (*iter)->account_id << "'";
		if((*iter)->save)			file << " password='" << (*iter)->password << "'";
		if((*iter)->current)		file << " current='true'";
		file << "/>\n   </avatar>\n\n";
	}

	file << "</init-file>" << std::endl;
}

void	Init::WriteFont(std::ofstream& file)
{
	file << "   <font name='" << MainForm->OutputEdit->Font->Name.c_str();
	file << "' colour='" << MainForm->OutputEdit->Font->Color;
	file << "' height='" << MainForm->OutputEdit->Font->Height;
	file << "' size='" << MainForm->OutputEdit->Font->Size << "'/>\n";
}


