/*-----------------------------------------------------------------------
				FedTerm version 1.60 - Windows Federation 2 Client
			  Copyright (c) 1995-2006 Interactive Broadcasting Ltd

	This program is free software: you can redistribute it and /or modify 
	it under the terms of the GNU General Public License as published by 
	the Free Software Foundation: either version 2 of the License, or (at 
	your option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY: without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
	General Public License for more details.

	You should have received a copy of the GNU General Public License 
	along with this program; if not, write to the Free Software Foundation, 
	Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307 USA

	Program written and compiled with Borland C++ Builder 5
	Programming and design: 	Alan Lenton
	Home website:					www.ibgames.net
-----------------------------------------------------------------------*/

#include <vcl.h>
#pragma hdrstop

#include <sstream>

#include <cstdlib>

#include "coms.h"
#include "fed2.h"
#include "friends_code.h"
#include "init_parser.h"
#include "init.h"
#include "login_code.h"
#include "macro.h"
#include "macro_rec.h"
#include "main.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------

const int	InitParser::NOT_AN_ELEMENT = -1;
const std::string	InitParser::elements[] =
{
	"avatar", "server", "account", "autolog", "macro", "command", 	//  0- 5
	"colour", "font", "buffer", "no-blank-lines", "size",          //  6-10
	"input-window", "stats-panel", "audio-alert", "twinkles",		// 11-14
	"friends-list", "friend", "voting", "desc2main", "sound-fx",	// 15-19
	""
};

InitParser::InitParser(const std::string& f_name) : XMLParser(f_name)
{
	Fed2::init = new Init(f_name);

	status = ANY;
	account = 0;
	macro_rec = 0;
	f_key = 0;
}

InitParser::~InitParser()
{
	// avoiding inline virtual destructors
}


void	InitParser::Account(const AttribList& attrib)
{
	FindAttrib(attrib,"id",account->account_id);
	FindAttrib(attrib,"password",account->password);
	account->save = (account->password != "");
	account->current = FindBoolAttrib(attrib,"current",false);
}

void	InitParser::AudioAlert(const AttribList& attrib)
{
	std::string	enabled;
	FindAttrib(attrib,"enabled",enabled);
	MainForm->TextAlertItem->Checked = (enabled == "true");
}

void	InitParser::Avatar(const AttribList& attrib)
{
	if(account != 0)
		delete account;
	account = new AccountRec;
	FindAttrib(attrib,"name",account->avatar_name);
}

void	InitParser::BackgroundColour(const AttribList& attrib)
{
	Fed2::init->NewBkgColour(static_cast<TColor>(FindNumAttrib(attrib,"bkg",clWindow)));
}

void	InitParser::EndAvatar()
{
	Fed2::init->AddAccount(account);
	account = 0;
}

void	InitParser::EndElement(const std::string& elem)
{
	switch(FindElement(elem))
	{
		case  0:	EndAvatar();		break;
		case	4: EndMacro();			break;
		case	5:	status = ANY;		break;
	}
}

void	InitParser::EndMacro()
{
	if(macro_rec != 0)
	{
		Fed2::macro->Add(f_key,macro_rec);
		macro_rec = 0;
		f_key = 0;
	}
}

int	InitParser::FindElement(const std::string& elem)
{
	for(int count = 0;elements[count] != "";count++)
	{
		if(elements[count] == elem)
			return(count);
	}
	return(NOT_AN_ELEMENT);
}

void	InitParser::Friend(const AttribList& attrib)
{
	std::string	name;
	FindAttrib(attrib,"name",name);
	Fed2::friends->AddFriend(name);
}

void	InitParser::FriendsList(const AttribList& attrib)
{
	std::string	ranks;
	FindAttrib(attrib,"ranks",ranks);
	unsigned long	rank_flags = std::atol(ranks.c_str());
	Fed2::friends->AddRanks(rank_flags);
	std::string	audio;
	FindAttrib(attrib,"audio",audio);
	if(audio == "true")
		Fed2::friends->SetAudio();
}

void	InitParser::InputWindow(const AttribList& attrib)
{
	int height = FindNumAttrib(attrib,"height",0);
	if(height > 0)
		MainForm->InputPanel->Height = height;
}

void	InitParser::MacroStart(const AttribList& attrib)
{
	macro_rec = new MacroRec;
	f_key = FindNumAttrib(attrib,"key",0);
}

void	InitParser::Server(const AttribList& attrib)
{
	FindAttrib(attrib,"host",account->host);
	account->port = FindNumAttrib(attrib,"port",30003);
}

void	InitParser::SetBuffer(const AttribList& attrib)
{
	int buffer_size = FindNumAttrib(attrib,"size",200);
	Fed2::init->SetBufferSize(buffer_size);
	MainForm->SetBufferSize(buffer_size);
}

void	InitParser::SetSoundFX(const AttribList& attrib)
{
	std::string	enabled;
	FindAttrib(attrib,"enabled",enabled);
	MainForm->SoundItem->Checked = (enabled == "true");
}

void	InitParser::SetTwinkles(const AttribList& attrib)
{
	std::string	enabled;
	FindAttrib(attrib,"enabled",enabled);
	MainForm->LightsItem->Checked = (enabled == "true");
}

void	InitParser::StartElement(const std::string& elem,const AttribList& attrib)
{
	switch(FindElement(elem))
	{
		case  0:	Avatar(attrib);									break;
		case	1:	Server(attrib);									break;
		case  2: Account(attrib);									break;
		case	3: Fed2::init->AutoLogging(true);				break;
		case	4: MacroStart(attrib);								break;
		case	5: status = CMD;										break;
		case	6: BackgroundColour(attrib);						break;
		case  7:	StartFont(attrib);								break;
		case  8:	SetBuffer(attrib);								break;
		case	9:	MainForm->BlankLinesItem->Checked = true;	break;
		case 10: WindowSize(attrib);								break;
		case 11:	InputWindow(attrib);								break;
		case 12: StatsPanel(attrib);								break;
		case 13: AudioAlert(attrib);								break;
		case 14:	SetTwinkles(attrib);								break;
		case 15:	FriendsList(attrib);								break;
		case 16:	Friend(attrib);									break;
		case 17: Voted(attrib);										break;
		case 19:	SetSoundFX(attrib);								break;
	}
}

void	InitParser::StartFont(const AttribList& attrib)
{
	TFont	*font = new TFont;
	std::string	name;

	FindAttrib(attrib,"name",name);
	font->Name = name.c_str();
	font->Color = static_cast<TColor>(FindNumAttrib(attrib,"colour",0));
	font->Height = FindNumAttrib(attrib,"height",-15);
	font->Pitch = fpDefault;
	font->Size = FindNumAttrib(attrib,"size",0);
	MainForm->SetFont(font);
}

void	InitParser::StatsPanel(const AttribList& attrib)
{
	std::string	visible;
	FindAttrib(attrib,"visible",visible);
	MainForm->SetStatsPanel(visible != "false");
}

void	InitParser::TextData(const std::string& text)
{
	if((status == CMD) && (macro_rec != 0))
		macro_rec->Add(text);
}

void	InitParser::Voted(const AttribList& attrib)
{
	std::string	vote_time;
	FindAttrib(attrib,"time",vote_time);
	Fed2::init->Voted(std::atol(vote_time.c_str()));
}

void	InitParser::WindowSize(const AttribList& attrib)
{
	int width = FindNumAttrib(attrib,"width",0);
	int height = FindNumAttrib(attrib,"height",0);
	if((width > 0) && (height > 0))
	{
		MainForm->Width = width;
		MainForm->Height = height;
	}
}


