/*-----------------------------------------------------------------------
				FedTerm version 1.60 - Windows Federation 2 Client
			  Copyright (c) 1995-2006 Interactive Broadcasting Ltd

	This program is free software: you can redistribute it and /or modify 
	it under the terms of the GNU General Public License as published by 
	the Free Software Foundation: either version 2 of the License, or (at 
	your option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY: without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
	General Public License for more details.

	You should have received a copy of the GNU General Public License 
	along with this program; if not, write to the Free Software Foundation, 
	Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307 USA

	Program written and compiled with Borland C++ Builder 5
	Programming and design: 	Alan Lenton
	Home website:					www.ibgames.net
-----------------------------------------------------------------------*/

#include <vcl.h>
#pragma hdrstop

#include <sstream>

#include "location.h"

#include "fedmap.h"
#include "main.h"
#include "xml.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------

const int Location::TILE_SIZE = 16;

const std::string	Location::exit_names[] =
{  "n", "ne", "e", "se", "s", "sw", "w", "nw", "u", "d"	};


Location::Location(const AttribList& attrib)
{
	XMLParser::FindAttrib(attrib,"name",name);
	int	loc_num = XMLParser::FindNumAttrib(attrib,"loc-num",-1);
	if((loc_num < 0) || (name == ""))
		x = y = -1;
	else
	{
		x = loc_num % FedMap::MAP_SIZE;
		y = loc_num / FedMap::MAP_SIZE;
	}

	std::string	temp;
	for(int count = 0;count < MAX_EXITS;count++)
	{
		XMLParser::FindAttrib(attrib,exit_names[count],temp);
		if(temp != "")
			exits.set(count);
	}
}


void	Location::Dump()
{
	std::ostringstream	buffer;
	buffer << "Location[" << x << "," << y << "]  (" << LocNum() << ")";
	buffer << " Exits :" << exits;
	MainForm->OutputDebug(buffer.str());
}

int	Location::LocNum()
{
	return(x + y * FedMap::MAP_SIZE);
}

void 	Location::Name2LocOutput()
{
	MainForm->OutputLoc(name);
}

void 	Location::Name2TextOutput()
{
	MainForm->OutputText(name,true);
}

void	Location::Write(std::ofstream& file)
{
	file << "   <loc name='" << XMLParser::EscapeXML(name) << "' loc-num='" << LocNum() << "'";
	for(int count = 0;count < MAX_EXITS;count++)
	{
		if(exits.test(count))
			file << " " << exit_names[count] << "='0'";
	}
	file << "/>\n";
}


