/*-----------------------------------------------------------------------
				FedTerm version 1.60 - Windows Federation 2 Client
			  Copyright (c) 1995-2006 Interactive Broadcasting Ltd

	This program is free software: you can redistribute it and /or modify 
	it under the terms of the GNU General Public License as published by 
	the Free Software Foundation: either version 2 of the License, or (at 
	your option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY: without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
	General Public License for more details.

	You should have received a copy of the GNU General Public License 
	along with this program; if not, write to the Free Software Foundation, 
	Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307 USA

	Program written and compiled with Borland C++ Builder 5
	Programming and design: 	Alan Lenton
	Home website:					www.ibgames.net
-----------------------------------------------------------------------*/

#include <vcl.h>
#pragma hdrstop

#include "login_code.h"

#include "fed2.h"
#include "init.h"
#include "logon.h"
#include "main.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------

Login::Login()
{
	status = ACCOUNT_ID;
	const AccountRec *rec = Fed2::init->GetCurrentAccount();
		account = new AccountRec;
	if(rec == 0)
	{
		account->host = "fed2.ibgames.net";
		account->port = 30003;
	}
	else
		UpdateLogonRecord(rec);
	UpdateLogonForm();
	LogonForm->ReplyTo(this);
	LogonForm->ShowModal();
}

Login::~Login()
{
	delete account;
}


int	Login::AccountID(const std::string& line)
{
	if((line.find("ogin:") != std::string::npos) ||
					(line.find("xisting account name:") != std::string::npos))
	{
		MainForm->Send(account->account_id + "\n");
		status = PASSWORD;
	}
	MainForm->OutputText(line);
	return(MORE_TO_COME);
}

void	Login::Connect(bool has_changed)
{
	if(has_changed)
	{
		account->host = LogonForm->HostEdit->Text.c_str();
		account->port = std::atoi(LogonForm->PortEdit->Text.c_str());
		account->avatar_name = LogonForm->FedEdit->Text.c_str();
		account->account_id = LogonForm->AccountEdit->Text.c_str();
		account->password = LogonForm->PasswordEdit->Text.c_str();
		account->current = true;
		account->save = LogonForm->SaveCheck->Checked;
		Fed2::init->UpdateAccount(account);
	}
	Fed2::init->Write();
	if(status == ACCOUNT_ID)
		MainForm->Logon(account->host,account->port,account->avatar_name);
	else
		status = ACCOUNT_ID;
	LogonForm->Close();
}

void	Login::DeleteCurrent()
{
	if(MessageDlg("Delete entry. Are you sure?",mtConfirmation,
										TMsgDlgButtons() << mbYes << mbNo,0) == mrNo)
		return;
	const AccountRec *rec = Fed2::init->DeleteCurrentAccount();
	if(rec == 0)
	{
		account->host = "fed2.ibgames.net";
		account->port = 30003;
	}
	else
		UpdateLogonRecord(rec);
	UpdateLogonForm();
}

void	Login::Next()
{
	const AccountRec *rec = Fed2::init->GetNextAccount();
	if(rec == 0)
	{
		account->avatar_name = "";
		account->account_id = "";
		account->password = "";
	}
	else
		UpdateLogonRecord(rec);
	UpdateLogonForm();
}

int	Login::Password(const std::string& line)
{
	if(line.find("assword:") != std::string::npos)
	{
		MainForm->Send(account->password + "\n");
		status = ALL_SENT;
	}
	MainForm->OutputText(line);
	return(MORE_TO_COME);
}

bool	Login::Previous()
{
	const AccountRec *rec = Fed2::init->GetPreviousAccount();
	if(rec == 0)
		return(false);
	else
		UpdateLogonRecord(rec);
	UpdateLogonForm();
	return(true);
}

int	Login::ProcessText(std::string& line)
{
	switch(status)
	{
		case ACCOUNT_ID:	return(AccountID(line));
		case PASSWORD:		return(Password(line));
		case ALL_SENT:		return(Start(line));
		default:				return(MORE_TO_COME);
	}
}

int	Login::Start(const std::string& line)
{
	static const std::string	error("Login Problem: The account ID and/or\npassword \
were incorrect!\nPlease correct and try again.");

	if(line.find("password is wrong") != std::string::npos)
	{
		MessageDlg(error.c_str(),mtInformation,TMsgDlgButtons() << mbOK,0);
		MainForm->OutputText(line);
		status = RETRY;
		MainForm->Send("no\n");
		LogonForm->ShowModal();
	}
	else
	{
		if(line.find("Linking to Federation DataSpace") != std::string::npos)
		{
			MainForm->OutputText(line);
			return(FINISHED);
		}
	}
	return(MORE_TO_COME);
}

void	Login::UpdateLogonForm()
{
	LogonForm->HostEdit->Text = account->host.c_str();
	LogonForm->PortEdit->Text = account->port;
	LogonForm->FedEdit->Text = account->avatar_name.c_str();
	LogonForm->AccountEdit->Text = account->account_id.c_str();
	LogonForm->PasswordEdit->Text = account->password.c_str();
	LogonForm->SaveCheck->Checked = account->save;
}

void	Login::UpdateLogonRecord(const AccountRec *rec)
{
	account->port = rec->port;
	account->avatar_name = rec->avatar_name;
	account->account_id = rec->account_id;
	account->password = rec->password;
	account->current = rec->current;
	account->save = rec->save;
	account->host = rec->host;
}


