/*-----------------------------------------------------------------------
				FedTerm version 1.60 - Windows Federation 2 Client
			  Copyright (c) 1995-2006 Interactive Broadcasting Ltd

	This program is free software: you can redistribute it and /or modify 
	it under the terms of the GNU General Public License as published by 
	the Free Software Foundation: either version 2 of the License, or (at 
	your option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY: without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
	General Public License for more details.

	You should have received a copy of the GNU General Public License 
	along with this program; if not, write to the Free Software Foundation, 
	Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307 USA

	Program written and compiled with Borland C++ Builder 5
	Programming and design: 	Alan Lenton
	Home website:					www.ibgames.net
-----------------------------------------------------------------------*/

#include <vcl.h>
#pragma hdrstop

#include "main.h"

#include <cstring>
#include <io.h>

#include <mmsystem.h>
#include <shellapi.h>
#include <winuser.h>

#include "about.h"
#include "avatar_stats.h"
#include "coms.h"
#include "cmdform.h"
#include "display.h"
#include "exchange.h"
#include "fed2.h"
#include "fedmap.h"
#include "friends.h"
#include "friends_code.h"
#include "init.h"
#include "init_parser.h"
#include "input.h"
#include	"logging.h"
#include "macro.h"
#include "macroform.h"
#include "players.h"
#include "ship_stats.h"
#include "tb_code.h"
#include "tmc_code.h"
#include "twinkles.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm *MainForm;
//---------------------------------------------------------------------------
  

const int TMainForm::ROW_COUNT = 8;

__fastcall TMainForm::TMainForm(TComponent* Owner)
	: TForm(Owner)
{
}

void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
	if(MessageDlg("Close Fedterm?",mtConfirmation,TMsgDlgButtons() << mbOK << mbCancel,0) != mrOk)
		Action = caNone;
}

void __fastcall TMainForm::FormCreate(TObject *Sender)
{
	buffer_size = 200;
	version = "1.60";
	char	f_buffer[MAXPATH];
	getcwd(f_buffer,MAXPATH);
	home_dir = f_buffer;
	if(access(".\\maps",0) != 0)
		mkdir(".\\maps");
	if(access(".\\logs",0) != 0)
		mkdir(".\\logs");
	if(access(".\\pictures",0) != 0)
		mkdir(".\\pictures");

	Fed2::friends = new Friends;
	Fed2::macro = new Macro;
	const std::string init_file = HomeDir() + "\\init.xml";
	InitParser *parser = new InitParser(init_file);
	parser->Parse();
	delete parser;
	Fed2::coms =  new Coms;
	Fed2::players =  new Players;
	Fed2::player_input = new PlayerInput;
	std::ostringstream buffer;
	buffer << "FedTerm - Version " << version;
	Caption = buffer.str().c_str();
	WinSocket = 0;
	if(Fed2::init->AutoLogging())
		AutoLogItemClick(this);
	StatsSetUp();
	Fed2::twinkles = new Twinkles;
	Fed2::tight_beam = new TightBeam;
	Fed2::tmc = new Tmc;
	Fed2::exchange= new Exchange;
}

void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
	TwinkleTimer->Enabled = false;
	TickerTimer->Enabled = false;
	Fed2::init->Write();
	delete Fed2::fed_map;
	delete Fed2::macro;
	delete Fed2::logging;
	delete Fed2::twinkles;
	delete Fed2::tight_beam;
	delete Fed2::friends;
	delete Fed2::tmc;
	delete Fed2::exchange;
}


void __fastcall TMainForm::AboutItemClick(TObject *Sender)
{
	AboutForm->Visible = true;
}

void	TMainForm::AddContents(const std::string& name)
{
	LocListBox->Items->Add(name.c_str());
}

void	TMainForm::AddPlayer(const std::string& name)
{
	PlayerListBox->Items->Add(name.c_str());
	DisplayPlayerNumbers();
}

void	TMainForm::AddFriend(const std::string& name)
{
	FriendsListBox->Items->Add(name.c_str());
}

void __fastcall TMainForm::AutoLogItemClick(TObject *Sender)
{
	if(!AutoLogItem->Checked)
	{
		if(Fed2::logging == 0)
			Fed2::logging = new Logging;

		if(Fed2::logging->IsLogging())
		{
			StartLoggingItem->Enabled = false;
			StopLoggingItem->Enabled = false;
			AutoLogItem->Checked = true;
			Fed2::init->AutoLogging(true);
		}
		else
		{
			delete Fed2::logging;
			Fed2::logging = 0;
		}
	}
	else
	{
		OutputText("Logging stopped");
		delete Fed2::logging;
		Fed2::logging = 0;
		StartLoggingItem->Enabled = true;
		StopLoggingItem->Enabled = false;
		AutoLogItem->Checked = false;
		Fed2::init->AutoLogging(false);
	}
}

void __fastcall TMainForm::BlankLinesItemClick(TObject *Sender)
{
	BlankLinesItem->Checked = !BlankLinesItem->Checked;
}

void __fastcall TMainForm::BoardBtnClick(TObject *Sender)
{
	WORD key = VK_ADD;
	if(Fed2::fed_map != 0)
		Fed2::fed_map->MovementKeys(key);

}

void __fastcall TMainForm::BufferSizeItemClick(TObject *Sender)
{
	BufferSizeItem->Checked = !BufferSizeItem->Checked;
	if(BufferSizeItem->Checked)
		buffer_size = 500;
	else
		buffer_size = 200;
	Fed2::init->SetBufferSize(buffer_size);
}

void	TMainForm::ClearAvatarAKCredits()
{
	ClearAvatarEntry(FindStat("Akaturi Cr"));
}

void	TMainForm::ClearAvatarEntry(int index)
{
	if(index >= 0)
	{
		StatsGrid->Cells[0][index] = "";
		StatsGrid->Cells[1][index] = "";
		ShuffleStatsDisplay();
	}
}

void	TMainForm::ClearAvatarHaulerCredits()
{
	ClearAvatarEntry(FindStat("Hauler Cr"));
}

void	TMainForm::ClearAvatarLoan()
{
	ClearAvatarEntry(FindStat("Loan"));
}

void	TMainForm::ClearAvatarMerchantCredits()
{
	ClearAvatarEntry(FindStat("Merchant Cr"));
}

void	TMainForm::ClearAvatarTraderCredits()
{
	ClearAvatarEntry(FindStat("Trader Cr"));
}

void	TMainForm::ClearAvatarShipDisplay()
{
	ClassLabel->Caption = "";
	if(access(".\\pictures\\space.bmp",0) == 0)
		ShipImage->Picture->LoadFromFile(".\\pictures\\space.bmp");
	for(int count = 0;count < ROW_COUNT;count++)
	{
		ShipGrid->Cells[0][count] = "";
		ShipGrid->Cells[1][count] = "";
	}
	UpdateShipStatBox("",0,100);
}

void	TMainForm::ClearAvatarStatsDisplay()
{
	NameLabel->Caption = "";
	RankLabel->Caption = "";
	if(access(".\\pictures\\interference.bmp",0) == 0)
		ScoreImage->Picture->LoadFromFile(".\\pictures\\interference.bmp");
	for(int count = 0;count < ROW_COUNT;count++)
	{
		StatsGrid->Cells[0][count] = "";
		StatsGrid->Cells[1][count] = "";
	}
	UpdateStatBox("",0,100);
}

void	TMainForm::ClearPlayers()
{
	PlayerListBox->Clear();
	StatusBar->Panels->Items[2]->Text = "";
}

void	TMainForm::ClipTextBuffer(TMemo *memo)
{
	// +100 so we don't update everytime there is a new line added
	int	num_lines = memo->Lines->Count;
	if(num_lines > (buffer_size + 100))
	{
		memo->Lines->BeginUpdate();
		for(int count = num_lines;count > buffer_size;count--)
			memo->Lines->Delete(0);
		memo->Lines->EndUpdate();
	}
}

void __fastcall TMainForm::CliSocketConnect(TObject *Sender,TCustomWinSocket *Socket)
{
	if(WinSocket == 0)
		WinSocket = Socket;
	LogonItem->Enabled = false;
	NewAccountAvatarItem->Enabled = false;
	LogOffItem->Enabled = true;
	Fed2::coms->Connected();
	std::ostringstream	buffer;
	buffer << "FedTerm - Version " << version << " - ";
	buffer << avatar_name << " logged into " << server_name;
	Caption =  buffer.str().c_str();
	FriendsListBox->Clear();
	if(Fed2::avatar_stats != 0)
		Fed2::avatar_stats->Clear();
	if(Fed2::ship_stats != 0)
		Fed2::ship_stats->Clear();
	if(LightsItem->Checked)
		Fed2::twinkles->Start();
}

void __fastcall TMainForm::CliSocketDisconnect(TObject *Sender,TCustomWinSocket *Socket)
{
	static std::string	disconnect("Disconnected");

	CliSocket->Close();
	OutputText(disconnect);
	LogonItem->Enabled = true;
	NewAccountAvatarItem->Enabled = true;
	LogOffItem->Enabled = false;
	StatusBar->Panels->Items[0]->Text = disconnect.c_str();
	std::ostringstream buffer;
	buffer << "FedTerm - Version " << version;
	Caption = buffer.str().c_str();
	if(LightsItem->Checked)
		Fed2::twinkles->AllOff();
	DetailGrid->Visible = false;
	MapImage->Visible = true;
}

void __fastcall TMainForm::CliSocketRead(TObject *Sender,
		TCustomWinSocket *Socket)
{
	static bool reading  = false;
	int	num_bytes;
	char	buffer[4096];

	if(reading)
		return;
	reading = true;
	while((num_bytes = Socket->ReceiveBuf(buffer,4095)) > 0)
	{
		buffer[num_bytes] = '\0';
		Fed2::coms->ServerText(buffer);
		if(DisplayServerStreamItem->Checked)
			ScrollBackMemo->Lines->Add(buffer);
	}
	reading = false;
}

void __fastcall TMainForm::ColorItemClick(TObject *Sender)
{
	ColorDialog->Color = OutputEdit->Color;
	if(ColorDialog->Execute())
	{
		Fed2::init->NewBkgColour(ColorDialog->Color);
		Fed2::init->Write();
	}
}

void __fastcall TMainForm::CommandBtnClick(TObject *Sender)
{
	CommandForm->Visible = true;
}

void __fastcall TMainForm::CopyItemClick(TObject *Sender)
{
	if(ActiveControl == OutputEdit)		{ OutputEdit->CopyToClipboard();			return;	}
	if(ActiveControl == InputEdit)		{ InputEdit->CopyToClipboard();			return;	}
	if(ActiveControl == StoryMemo)		{ StoryMemo->CopyToClipboard();			return;	}
	if(ActiveControl == ScrollBackMemo)	{ ScrollBackMemo->CopyToClipboard();	return;	}
}

void	TMainForm::DeletePlayer(const std::string& name)
{
	int index = PlayerListBox->Items->IndexOf(name.c_str());
	if(index >= 0)
		PlayerListBox->Items->Delete(index);
	DisplayPlayerNumbers();
}

void	TMainForm::DeleteFriend(const std::string& name)
{
	int index = FriendsListBox->Items->IndexOf(name.c_str());
	if(index >= 0)
		FriendsListBox->Items->Delete(index);
}
void __fastcall TMainForm::DetailGridDrawCell(TObject *Sender, int ACol,
		int ARow, TRect &Rect, TGridDrawState State)
{
	if(Fed2::fed_map != 0)
		Fed2::fed_map->DrawCell(ACol,ARow,Rect,DetailGrid);

}void __fastcall TMainForm::DetailGridKeyUp(TObject *Sender, WORD &Key,
		TShiftState Shift)
{
	if(Fed2::fed_map != 0)
		Fed2::fed_map->MovementKeys(Key);
}

void	TMainForm::DisplayAvatarAKCredits(int ak)
{
	int	index = FindStat("Akaturi Cr");
	if(index >= 0)
	{
		StatsGrid->Cells[0][index] = "Akaturi Cr";
		StatsGrid->Cells[1][index] = ak;
		if(StatsGrid->Row == index)
			UpdateStatBox("",0,100);
	}
}

void	TMainForm::DisplayAvatarCash(const std::string& amount)
{
	int	index = FindStat("Cash");
	if(index >= 0)
	{
		StatsGrid->Cells[0][index] = "Cash";
		StatsGrid->Cells[1][index] = amount.c_str();
		if(StatsGrid->Row == index)
			UpdateStatBox("",0,100);
	}
}

void	TMainForm::DisplayAvatarDexterity(int maximum,int current)
{
	std::ostringstream	buffer;
	buffer << current << "/" << maximum;
	int	index = FindStat("Dexterity");
	if(index >= 0)
	{
		StatsGrid->Cells[0][index] = "Dexterity";
		StatsGrid->Cells[1][index] = buffer.str().c_str();
		if(StatsGrid->Row == index)
			UpdateStatBox(" Dexterity ",current,maximum);
	}
}

void	TMainForm::DisplayAvatarHaulerCredits(int hc)
{
	int	index = FindStat("Hauler Cr");
	if(index >= 0)
	{
		StatsGrid->Cells[0][index] = "Hauler Cr";
		StatsGrid->Cells[1][index] = hc;
		if(StatsGrid->Row == index)
			UpdateStatBox("",0,100);
	}
}

void	TMainForm::DisplayAvatarIntelligence(int maximum,int current)
{
	std::ostringstream	buffer;
	buffer << current << "/" << maximum;
	int	index = FindStat("Intelligence");
	if(index >= 0)
	{
		StatsGrid->Cells[0][index] = "Intelligence";
		StatsGrid->Cells[1][index] = buffer.str().c_str();
		if(StatsGrid->Row == index)
			UpdateStatBox(" Intelligence ",current,maximum);
	}
}

void	TMainForm::DisplayAvatarLoan(const std::string& amount)
{
	int	index = FindStat("Loan");
	if(index >= 0)
	{
		StatsGrid->Cells[0][index] = "Loan";
		StatsGrid->Cells[1][index] = amount.c_str();
		if(StatsGrid->Row == index)
			UpdateStatBox("",0,100);
	}
}

void	TMainForm::DisplayAvatarMerchantCredits(int merchant_pts)
{
	int	index = FindStat("Merchant Cr");
	if(index >= 0)
	{
		StatsGrid->Cells[0][index] = "Merchant Cr";
		StatsGrid->Cells[1][index] = merchant_pts;
		if(StatsGrid->Row == index)
			UpdateStatBox("",0,100);
	}
}

void	TMainForm::DisplayAvatarName(const std::string& name)
{
	NameLabel->Caption = name.c_str();
	std::string	temp(name);
	temp[0] = std::tolower(temp[0]);
	std::string	file_name = ".\\pictures\\" + temp + ".bmp";
	if(access(file_name.c_str(),0) != 0)
	{
		file_name = ".\\pictures\\interference.bmp";
		if(access(file_name.c_str(),0) != 0)
			return;
	}
	ScoreImage->Picture->LoadFromFile(file_name.c_str());
}

void	TMainForm::DisplayAvatarRank(const std::string& rank)
{
	RankLabel->Caption = rank.c_str();
}

void	TMainForm::DisplayAvatarStamina(int maximum,int current)
{
	std::ostringstream	buffer;
	buffer << current << "/" << maximum;
	int	index = FindStat("Stamina");
	if(index >= 0)
	{
		StatsGrid->Cells[0][index] = "Stamina";
		StatsGrid->Cells[1][index] = buffer.str().c_str();
		if(StatsGrid->Row == index)
			UpdateStatBox(" Stamina ",current,maximum);
	}
}

void	TMainForm::DisplayAvatarStrength(int maximum,int current)
{
	std::ostringstream	buffer;
	buffer << current << "/" << maximum;
	int	index = FindStat("Strength");
	if(index >= 0)
	{
		StatsGrid->Cells[0][index] = "Strength";
		StatsGrid->Cells[1][index] = buffer.str().c_str();
		if(StatsGrid->Row == index)
			UpdateStatBox(" Strength ",current,maximum);
	}
}

void	TMainForm::DisplayAvatarTraderCredits(int trader_pts)
{
	int	index = FindStat("Trader Cr");
	if(index >= 0)
	{
		StatsGrid->Cells[0][index] = "Trader Cr";
		StatsGrid->Cells[1][index] = trader_pts;
		if(StatsGrid->Row == index)
			UpdateStatBox("",0,100);
	}
}

void __fastcall TMainForm::DisplayClientStreamItemClick(TObject *Sender)
{
	DisplayClientStreamItem->Checked = !DisplayClientStreamItem->Checked;
}

void	TMainForm::DisplayPlayerNumbers()
{
	std::ostringstream	buffer;
	buffer << PlayerListBox->Items->Count << " players";
	StatusBar->Panels->Items[2]->Text = buffer.str().c_str();
}

void	TMainForm::DisplayScoreStats()
{
	StatsNoteBook->ActivePage = ScoreTab;
}

void __fastcall TMainForm::DisplayServerStreamItemClick(TObject *Sender)
{
	DisplayServerStreamItem->Checked = !DisplayServerStreamItem->Checked;

}

void	TMainForm::DisplayShipClass(const std::string& class_name)
{
	std::string	name = "-" + class_name + " Class-";
	ClassLabel->Caption = name.c_str();
	std::string	temp(class_name);
	temp[0] = std::tolower(temp[0]);
	std::string	file_name = ".\\pictures\\" + temp + ".bmp";
	if(access(file_name.c_str(),0) != 0)
	{
		file_name = ".\\pictures\\space.bmp";
		if(access(file_name.c_str(),0) != 0)
			return;
	}
	ShipImage->Picture->LoadFromFile(file_name.c_str());
}

void	TMainForm::DisplayShipCargo(int maximum,int current)
{
	std::ostringstream	buffer;
	buffer << current << "/" << maximum;
	int	index = FindShipStat("Cargo Space");
	if(index >= 0)
	{
		ShipGrid->Cells[0][index] = "Cargo Space";
		ShipGrid->Cells[1][index] = buffer.str().c_str();
		if(ShipGrid->Row == index)
			UpdateShipStatBox(" Cargo Space Left ",current,maximum);
//			UpdateShipStatBox("",0,100);
	}
}

void	TMainForm::DisplayShipComputer(int maximum,int current)
{
	std::ostringstream	buffer;
	buffer << current << "/" << maximum;
	int	index = FindShipStat("Computer");
	if(index >= 0)
	{
		ShipGrid->Cells[0][index] = "Computer";
		ShipGrid->Cells[1][index] = buffer.str().c_str();
		if(ShipGrid->Row == index)
			UpdateShipStatBox(" Computer ",current,maximum);
	}
}

void	TMainForm::DisplayShipCustomsCert()
{
	int	index = FindShipStat("Customs");
	if(index >= 0)
	{
		ShipGrid->Cells[0][index] = "Customs";
		ShipGrid->Cells[1][index] = "Yes";
		if(ShipGrid->Row == index)
			UpdateShipStatBox("",0,0);
	}
}

void	TMainForm::DisplayShipEngines(int maximum,int current)
{
	std::ostringstream	buffer;
	buffer << current << "/" << maximum;
	int	index = FindShipStat("Engines");
	if(index >= 0)
	{
		ShipGrid->Cells[0][index] = "Engines";
		ShipGrid->Cells[1][index] = buffer.str().c_str();
		if(ShipGrid->Row == index)
			UpdateShipStatBox(" Engines ",current,maximum);
	}
}

void	TMainForm::DisplayShipFuel(int maximum,int current)
{
	std::ostringstream	buffer;
	buffer << current << "/" << maximum;
	int	index = FindShipStat("Fuel");
	if(index >= 0)
	{
		ShipGrid->Cells[0][index] = "Fuel";
		ShipGrid->Cells[1][index] = buffer.str().c_str();
		if(ShipGrid->Row == index)
			UpdateShipStatBox(" Fuel ",current,maximum);
	}
}

void	TMainForm::DisplayShipHull(int maximum,int current)
{
	std::ostringstream	buffer;
	buffer << current << "/" << maximum;
	int	index = FindShipStat("Hull");
	if(index >= 0)
	{
		ShipGrid->Cells[0][index] = "Hull";
		ShipGrid->Cells[1][index] = buffer.str().c_str();
		if(ShipGrid->Row == index)
			UpdateShipStatBox(" Hull ",current,maximum);
	}
}

void	TMainForm::DisplayShipNavComp()
{
	int	index = FindShipStat("Nav Comp");
	if(index >= 0)
	{
		ShipGrid->Cells[0][index] = "Nav Comp";
		ShipGrid->Cells[1][index] = "Installed";
		if(ShipGrid->Row == index)
			UpdateShipStatBox("",0,0);
	}
}

void	TMainForm::DisplayShipShields(int maximum,int current)
{
	std::ostringstream	buffer;
	buffer << current << "/" << maximum;
	int	index = FindShipStat("Shields");
	if(index >= 0)
	{
		ShipGrid->Cells[0][index] = "Shields";
		ShipGrid->Cells[1][index] = buffer.str().c_str();
		if(ShipGrid->Row == index)
			UpdateShipStatBox(" Shields ",current,maximum);
	}
}

void	TMainForm::DisplayShipStats()
{
	StatsNoteBook->ActivePage = ShipTab;
}

void __fastcall TMainForm::DownBtnMouseUp(TObject *Sender,
		TMouseButton Button, TShiftState Shift, int X, int Y)
{
	WORD key = VK_INSERT;
	if(Fed2::fed_map != 0)
		Fed2::fed_map->MovementKeys(key);
	if(MainForm->MainNotebook->ActivePage == MainForm->MainTab)
		MainForm->ActiveControl = MainForm->InputEdit;
}

void __fastcall TMainForm::DumptoScrollItemClick(TObject *Sender)
{
	MainNotebook->ActivePage = MainTab;
	OutputEdit->SelectAll();
	OutputEdit->CopyToClipboard();
	OutputEdit->SetFocus();
	ScrollBackMemo->ReadOnly = false;
	ScrollBackMemo->Clear();
	ScrollBackMemo->PasteFromClipboard();
	ScrollBackMemo->ReadOnly = true;
	MainNotebook->ActivePage = ScrollTab;
}

void	TMainForm::Emote(const char *emotion)
{
	TListBox	*list_box;
	if(PlayersNotebook->ActivePage == PlayersTab)
		list_box = PlayerListBox;
	else
		list_box = FriendsListBox;
	if(list_box->ItemIndex >= 0)
	{
		std::ostringstream	buffer;
		buffer << emotion << " " << list_box->Items->Strings[list_box->ItemIndex].c_str();
		OutputText(buffer.str());
		buffer << "\n";
		Send(buffer.str());
	}
	else
		MessageDlg("You need to select a player first!",mtError,TMsgDlgButtons() << mbOK,0);
}

void __fastcall TMainForm::ExamineItemClick(TObject *Sender)
{
	Emote("ex");
}

void	TMainForm::ExchangeTicker(const std::string& text)
{
	static const std::string blanks("                                                                          ");
	static std::string	buffer(blanks);
	static int end = 0;
	static int index = 0;
	buffer += text;
	if(end == 0)
		end = buffer.length();
	buffer.erase(0,1);
	TickerEdit->Text = buffer.c_str();
	if(index == 0)
		TickerTimer->Enabled = true;
	if(++index >= end)
	{
		TickerTimer->Enabled = false;
		end = index = 0;
		buffer = blanks;
	}
}

void __fastcall TMainForm::ExitItemClick(TObject *Sender)
{
	Close();
}

void __fastcall TMainForm::ExtrasItemClick(TObject *Sender)
{
	ShellExecute(Handle,0,"http://www.ibgames.net/fed2/extras/index.html",0,0,SW_NORMAL);
}
void __fastcall TMainForm::FedTermManual1Click(TObject *Sender)
{
	ShellExecute(Handle,0,"http://www.ibgames.net/fed2/fedterm/manual/index.html",0,0,SW_NORMAL);
}

void __fastcall TMainForm::FeedbackItemClick(TObject *Sender)
{
	ShellExecute(Handle,0,"mailto:feedback@ibgames.com",0,0,SW_NORMAL);
}

int	TMainForm::FindPlayer(const std::string& name)
{
	return(PlayerListBox->Items->IndexOf(name.c_str()));
}

int	TMainForm::FindShipStat(const std::string& stat_name)
{
	for(int count = 0;count < ShipGrid->RowCount;count++)
	{
		if((ShipGrid->Cells[0][count] == stat_name.c_str()) || (ShipGrid->Cells[0][count] == ""))
			return(count);
	}
	return(-1);
}

int	TMainForm::FindStat(const std::string& stat_name)
{
	for(int count = 0;count < StatsGrid->RowCount;count++)
	{
		if((StatsGrid->Cells[0][count] == stat_name.c_str()) || (StatsGrid->Cells[0][count] == ""))
			return(count);
	}
	return(-1);
}

void __fastcall TMainForm::FKeyStartUpItemClick(TObject *Sender)
{
	Fed2::macro->Process();;
}

void __fastcall TMainForm::FontsItemClick(TObject *Sender)
{
	FontDialog->Font = OutputEdit->Font;
	if(FontDialog->Execute())
	{
		SetFont(FontDialog->Font);
		Fed2::init->Write();
	}
}

void __fastcall TMainForm::FormResize(TObject *Sender)
{
	TickerEdit->Left = (TickerPanel->Width - TickerEdit->Width)/2;
}

void __fastcall TMainForm::FormShow(TObject *Sender)
{
	static bool first_time = true;

	if(first_time)
	{
		first_time = false;
		Fed2::tmc->CheckTime();
	}
}

void __fastcall TMainForm::FriendsitemClick(TObject *Sender)
{
	FriendsForm->Visible = true;
}

void __fastcall TMainForm::HistoryListBoxClick(TObject *Sender)
{
	int index = HistoryListBox->ItemIndex;
	if(index >= 0)
		HistoryListBox->Hint = HistoryListBox->Items->Strings[index];
}

void __fastcall TMainForm::HistoryListBoxDblClick(TObject *Sender)
{
	if(HistoryListBox->ItemIndex == -1)
		return;
	std::string	which = HistoryListBox->Items->Strings[HistoryListBox->ItemIndex].c_str();
	Fed2::exchange->Transaction(which);
}

void __fastcall TMainForm::HugItemClick(TObject *Sender)
{
	Emote("hug");
}

void __fastcall TMainForm::InputEditKeyUp(TObject *Sender, WORD &Key,
		TShiftState Shift)
{
	switch(Key)
	{
		case VK_RETURN:	Fed2::player_input->Process(InputEdit->Text.c_str());	break;
		case VK_SPACE:
			if(Shift.Contains(ssShift) && Shift.Contains(ssCtrl) && (PlayerListBox->ItemIndex >= 0))
			{
				std::ostringstream	buffer;
				buffer << InputEdit->Text.c_str();
				buffer << " " << PlayerListBox->Items->Strings[PlayerListBox->ItemIndex].c_str() << " ";
				InputEdit->Text = buffer.str().c_str();
				InputEdit->SelStart = buffer.str().length();
				ActiveControl = InputEdit;
			}
			if(Shift.Contains(ssCtrl) && !Shift.Contains(ssShift) && (PlayerListBox->ItemIndex >= 0))
			{
				InputEdit->Clear();
 				std::ostringstream	buffer;
				buffer << InputEdit->Text.c_str();
				buffer << "Tell " << PlayerListBox->Items->Strings[PlayerListBox->ItemIndex].c_str() << " ";
				InputEdit->Text = buffer.str().c_str();
				InputEdit->SelStart = buffer.str().length();
				ActiveControl = InputEdit;
			}
			break;

			case VK_F1:
			case VK_F2:
			case VK_F3:
			case VK_F4:
			case VK_F5:
			case VK_F6:
			case VK_F7:
			case VK_F8:
			case VK_F9:
			case VK_F10:
			case VK_F11:
			case VK_F12:	if(Shift.Contains(ssCtrl))
									Fed2::macro->Transfer(1 + Key - VK_F1,InputEdit);
								else
									Fed2::macro->Send(1 + Key - VK_F1);
								break;
	}
}

void __fastcall TMainForm::InputEditEnter(TObject *Sender)
{
	PasteItem->Enabled = true;
}

void __fastcall TMainForm::InputEditExit(TObject *Sender)
{
	PasteItem->Enabled = false;

}

void __fastcall TMainForm::KissItemClick(TObject *Sender)
{
	Emote("kiss");
}

void __fastcall TMainForm::LightsItemClick(TObject *Sender)
{
	LightsItem->Checked = !LightsItem->Checked;
	if(LightsItem->Checked)
		Fed2::twinkles->Start();
	else
		Fed2::twinkles->AllOff();
}

void __fastcall TMainForm::LocListBoxDblClick(TObject *Sender)
{
	if(LocListBox->ItemIndex >= 0)
	{
		std::ostringstream	buffer;
		buffer << "ex " << LocListBox->Items->Strings[LocListBox->ItemIndex].c_str();
		Fed2::player_input->Process(buffer.str().c_str(),false);
		ActiveControl = InputEdit;
	}
}

void __fastcall TMainForm::LogOffItemClick(TObject *Sender)
{
	const char *disconnect = "Disconnected";

	CliSocket->Close();

	NewAccountAvatarItem->Enabled = true;
	LogonItem->Enabled = true;
	LogOffItem->Enabled = false;

	StatusBar->Panels->Items[0]->Text = disconnect;
	OutputText("Connection to Federation II Closed");
}

void	TMainForm::Logon(std::string& host,int port_num,std::string& pl_name)
{
	if(Fed2::fed_map != 0)
		Fed2::fed_map->Clear();
	Fed2::players->Clear();
	try
	{
		CliSocket->Host = host.c_str();
		CliSocket->Port = port_num;
		CliSocket->Open();
	} catch(...) { SocketOpenException(); }
	avatar_name = pl_name;
	server_name = host;
}

void __fastcall TMainForm::LogonItemClick(TObject *Sender)
{
	Fed2::coms->Logon();
}

void __fastcall TMainForm::MainNotebookChange(TObject *Sender)
{
	if(MainNotebook->ActivePage == MainTab)
	{
		OutputText(" ",true);
		ActiveControl = InputEdit;
		return;
	}
	if(MainNotebook->ActivePage == MapTab)
	{
		ClipTextBuffer(MapRepeater);
		MapRepeater->Lines->Add(" ");
		return;
	}
	if(MainNotebook->ActivePage == StoryTab)
	{
		ClipTextBuffer(StoryMemo);
		StoryMemo->Lines->Add(" ");
		return;
	}
}

void __fastcall TMainForm::ManualsItemClick(TObject *Sender)
{
	ShellExecute(Handle,0,"http://www.ibgames.net/fed2/guide/index.html",0,0,SW_NORMAL);
}

void __fastcall TMainForm::MapGridDrawCell(TObject *Sender, int ACol,
		int ARow, TRect &Rect, TGridDrawState State)
{
	if(Fed2::fed_map != 0)
		Fed2::fed_map->DrawCell(ACol,ARow,Rect,MapGrid);
}

void __fastcall TMainForm::MapGridKeyUp(TObject *Sender, WORD &Key,
		TShiftState Shift)
{
	if(Fed2::fed_map != 0)
		Fed2::fed_map->MovementKeys(Key);
}

void __fastcall TMainForm::MapsItemClick(TObject *Sender)
{
	ShellExecute(Handle,0,"http://www.ibgames.net/fed2/maps/planetmaps.html",0,0,SW_NORMAL);
}

void __fastcall TMainForm::MapMouseMove(TObject *Sender,TShiftState Shift, int X, int Y)
{
	int	col,row;
	static_cast<TDrawGrid *>(Sender)->MouseToCell(X,Y,col,row);
	if(Fed2::fed_map != 0)
		Fed2::fed_map->Name2Status(col,row);
}

void __fastcall TMainForm::MoveBtnClick(TObject *Sender)
{
	WORD key = static_cast<TSpeedButton *>(Sender)->Tag  + VK_PRIOR;
	if(Fed2::fed_map != 0)
		Fed2::fed_map->MovementKeys(key);
}

void __fastcall TMainForm::NewAccountAvatarItemClick(TObject *Sender)
{
	Fed2::coms->NewAccountAvatar();
}

void	TMainForm::OutputDebug(const char *text)
{
	ClipTextBuffer(ScrollBackMemo);
	ScrollBackMemo->Lines->Add(text);
}

void	TMainForm::OutputLoc(const char *text)
{
	ClipTextBuffer(StoryMemo);
	OutputStory(text);
}

void	TMainForm::OutputStory(const char *text)
{
	ClipTextBuffer(StoryMemo);
	StoryMemo->Lines->Add(text);
	if(Fed2::logging != 0)
		Fed2::logging->Write(text);
}

void	TMainForm::OutputText(const char *text,bool no_story)
{
	if(BlankLinesItem->Checked && ((strlen(text) == 0) || (strcmp(text," ") == 0)))
		return;

	ClipTextBuffer(OutputEdit);
	OutputEdit->Lines->Add(text);
	ClipTextBuffer(MapRepeater);
	MapRepeater->Lines->Add(text);
	if(!no_story)
		OutputStory(text);
	if((TextAlertItem->Checked) && (text[0] != '>'))
	{
		std::string buffer = HomeDir() + "\\sounds\\alert.wav";
		sndPlaySound(buffer.c_str(),SND_ASYNC);
	}
}

void __fastcall TMainForm::PasteItemClick(TObject *Sender)
{
	// The only thing allowable for pasting is the input buffer
	InputEdit->PasteFromClipboard();
}

void __fastcall TMainForm::PlayerListBoxDblClick(TObject *Sender)
{
	if(PlayerListBox->ItemIndex >= 0)
	{
		std::ostringstream	buffer;
		buffer << InputEdit->Text.c_str();
		buffer << " " << PlayerListBox->Items->Strings[PlayerListBox->ItemIndex].c_str() << " ";
		InputEdit->Text = buffer.str().c_str();
		InputEdit->SelStart = buffer.str().length();
		ActiveControl = InputEdit;
	}
}

void __fastcall TMainForm::QuickStartItemClick(TObject *Sender)
{
	ShellExecute(Handle,0,"http://www.ibgames.net/fed2/manuals/quick.html",0,0,SW_NORMAL);
}

void	TMainForm::RemoveContents(const std::string& name)
{
	int index = LocListBox->Items->IndexOf(name.c_str());
	if(index >= 0)
		LocListBox->Items->Delete(index);
}

void __fastcall TMainForm::SaveScreenItemClick(TObject *Sender)
{
	Logging::ScreenDump();
}

void __fastcall TMainForm::SelectAllItemClick(TObject *Sender)
{
	if(ActiveControl == OutputEdit)		{ OutputEdit->SelectAll();	return;	}
	if(ActiveControl == InputEdit)		{ InputEdit->SelectAll();	return;	}
	if(ActiveControl == StoryMemo)		{ StoryMemo->SelectAll();	return;	}
	if(ActiveControl == ScrollBackMemo)	{ ScrollBackMemo->SelectAll();	return;	}
}

void	TMainForm::Send(const std::string& text)
{
	if(WinSocket != 0)
	{
		WinSocket->SendBuf(const_cast<void *>(static_cast<const void *>(text.c_str())),text.length());
		if(DisplayClientStreamItem->Checked)
			ScrollBackMemo->Lines->Add(text.c_str());
	}
}

void	TMainForm::SetBufferSize(int new_size)
{
	buffer_size = new_size;
	BufferSizeItem->Checked = (buffer_size > 200);
}

void	TMainForm::SetColour(TColor colour)
{
	InputEdit->Color = colour;
	OutputEdit->Color = colour;
	LocListBox->Color = colour;
	PlayerListBox->Color = colour;
	FriendsListBox->Color = colour;
	MapMemo->Color = colour;
	MapRepeater->Color = colour;
	StoryMemo->Color = colour;
	ScrollBackMemo->Color = colour;
	StatsGrid->Color = colour;
	ShipGrid->Color = colour;
	TBNamePanel->Color = colour;
	TBMemo->Color = colour;
	HistoryListBox->Color = colour;
	TickerEdit->Color = colour;
	LastCommodPanel->Color = colour;
	SpynetOutput->Color = colour;
	SpynetNamePanel->Color = colour;
}

void	TMainForm::SetFont(TFont *font)
{
	InputEdit->Font = font;
	OutputEdit->Font = font;
	LocListBox->Font = font;
	PlayerListBox->Font = font;
	FriendsListBox->Font = font;
	MapMemo->Font = font;
	MapRepeater->Font = font;
	StoryMemo->Font = font;
	ScrollBackMemo->Font = font;
	TBNamePanel->Font = font;
	TBMemo->Font = font;
	HistoryListBox->Font = font;
	SpynetOutput->Font = font;
	SpynetNamePanel->Font = font;

	StatsGrid->Font->Color = font->Color;
	ShipGrid->Font->Color = font->Color;
	CommodNameLabel->Font->Color = font->Color;
	CommodLabel1->Font->Color = font->Color;
	CommodLabel2->Font->Color = font->Color;
	TickerEdit->Font->Color = font->Color;
}

void	TMainForm::SetSpynetOutputToTop()
{
	SpynetOutput->Lines->Insert(0," ");
	SpynetOutput->Lines->Delete(0);
}

void	TMainForm::SetStatsPanel(bool visible)
{
	StatsItem->Checked = visible;
	StatsPanel->Visible = visible;
}

void __fastcall TMainForm::ShipGridClick(TObject *Sender)
{
	std::string	stat_name((ShipGrid->Cells[0][ShipGrid->Row]).c_str());
	Fed2::ship_stats->NewStatSelect(stat_name);

}

void	TMainForm::ShuffleStatsDisplay()
{
	for(int count = 0;count < (ROW_COUNT - 1);count++)
	{
		if((StatsGrid->Cells[0][count] == "") && (StatsGrid->Cells[0][count + 1] != ""))
		{
			StatsGrid->Cells[0][count] = StatsGrid->Cells[0][count + 1];
			StatsGrid->Cells[1][count] = StatsGrid->Cells[1][count + 1];
			StatsGrid->Cells[0][count + 1] = "";
			StatsGrid->Cells[1][count + 1] = "";
		}
	}
}

void	TMainForm::SocketOpenException()
{
	static std::string disconnect("Disconnected");

	StatusBar->Panels->Items[0]->Text = disconnect.c_str();
	OutputText("   ");
	OutputText("Your computer has reported an internet connection error.");
	OutputText("This may be because your comm settings are wrong,");
	OutputText("because of serious network delays, or because the");
	OutputText("Federation Server computer is temporarily unavailable.");
}

void __fastcall TMainForm::SoundItemClick(TObject *Sender)
{
	SoundItem->Checked = !SoundItem->Checked;
}

void __fastcall TMainForm::SpynetReportClick(TObject *Sender)
{
	Emote("spynet report");
}

void __fastcall TMainForm::StartLoggingItemClick(TObject *Sender)
{
	OpenDialog->Title = "Log to file...";
	std::string	dir = HomeDir() + "\\logs";
	OpenDialog->InitialDir = dir.c_str();
	if(OpenDialog->Execute())
	{
		std::string	file_name(OpenDialog->FileName.c_str());
		Fed2::logging = new Logging(file_name);
		if(Fed2::logging->IsLogging())
		{
			StartLoggingItem->Enabled = false;
			StopLoggingItem->Enabled = true;
			AutoLogItem->Checked = false;
		}
		else
		{
			delete Fed2::logging;
			Fed2::logging = 0;
		}
	}
}

void __fastcall TMainForm::StatsGridClick(TObject *Sender)
{
	std::string	stat_name((StatsGrid->Cells[0][StatsGrid->Row]).c_str());
	Fed2::avatar_stats->NewStatSelect(stat_name);
}

void __fastcall TMainForm::StatsItemClick(TObject *Sender)
{
	StatsItem->Checked = !StatsItem->Checked;
	StatsPanel->Visible = StatsItem->Checked;
}

void	TMainForm::StatsSetUp()
{
	StatsGrid->ColWidths[0] = 88;
	StatsGrid->ColWidths[1] = 60;
	ShipGrid->ColWidths[0] = 88;
	ShipGrid->ColWidths[1] = 60;
}

void __fastcall TMainForm::StopLoggingItemClick(TObject *Sender)
{
	OutputText("Logging stopped");
	delete Fed2::logging;
	Fed2::logging = 0;
	StartLoggingItem->Enabled = true;
	StopLoggingItem->Enabled = false;
	AutoLogItem->Checked = false;
	Fed2::init->AutoLogging(false);
}

void __fastcall TMainForm::TBEndBtnClick(TObject *Sender)
{
	Fed2::tight_beam->FinishedWithPlayer();
}
void __fastcall TMainForm::TextAlertItemClick(TObject *Sender)
{
	TextAlertItem->Checked = !TextAlertItem->Checked;
}

void __fastcall TMainForm::TickerTimerTimer(TObject *Sender)
{
	static std::string blank("");
	ExchangeTicker(blank);
}

void __fastcall TMainForm::TightBeamItemClick(TObject *Sender)
{
	TListBox	*list_box;
	if(PlayersNotebook->ActivePage == PlayersTab)
		list_box = PlayerListBox;
	else
		list_box = FriendsListBox;
	if(list_box->ItemIndex >= 0)
		Fed2::tight_beam->NewPlayer(list_box->Items->Strings[list_box->ItemIndex].c_str());
	else
		MessageDlg("You haven't selected\na player to talk to!\n",mtError,TMsgDlgButtons() << mbOK,0);
}

void __fastcall TMainForm::TopNotebookChange(TObject *Sender)
{
	if(TopNotebook->ActivePage == ExchTab)
		TickerEdit->Left = (TickerPanel->Width - TickerEdit->Width)/2;
}

void __fastcall TMainForm::TwinkleTimerTimer(TObject *Sender)
{
	Fed2::twinkles->Random();
}

void __fastcall TMainForm::UpBtnMouseUp(TObject *Sender,
		TMouseButton Button, TShiftState Shift, int X, int Y)
{
	WORD key = VK_DELETE;
	if(Fed2::fed_map != 0)
		Fed2::fed_map->MovementKeys(key);
	if(MainForm->MainNotebook->ActivePage == MainForm->MainTab)
		MainForm->ActiveControl = MainForm->InputEdit;
}

void	TMainForm::UpdateShipStatBox(const std::string& stat,int current,int maximum)
{
	ShipBox->Caption = stat.c_str();
	int position = (current * 100)/maximum;
	ShipBar->Position = position;
	if((position == 0) || (position > 66))
		ShipBar->Brush->Color = clBtnFace;
	else
	{
		if(position > 33)
			ShipBar->Brush->Color = clYellow;
		else
			ShipBar->Brush->Color = clRed;
	}
	ShipBar->Repaint();
}

void	TMainForm::UpdateStatBox(const std::string& stat,int current,int maximum)
{
	StatBox->Caption = stat.c_str();
	int position = (current * 100)/maximum;
	StatBar->Position = position;
	if((position == 0) || (position > 66))
		StatBar->Brush->Color = clBtnFace;
	else
	{
		if(position > 33)
			StatBar->Brush->Color = clYellow;
		else
			StatBar->Brush->Color = clRed;
	}
	StatBar->Repaint();
}

void __fastcall TMainForm::VotingItemClick(TObject *Sender)
{
	Fed2::tmc->MenuClick();
}






