/*-----------------------------------------------------------------------
				FedTerm version 1.60 - Windows Federation 2 Client
			  Copyright (c) 1995-2006 Interactive Broadcasting Ltd

	This program is free software: you can redistribute it and /or modify 
	it under the terms of the GNU General Public License as published by 
	the Free Software Foundation: either version 2 of the License, or (at 
	your option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY: without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
	General Public License for more details.

	You should have received a copy of the GNU General Public License 
	along with this program; if not, write to the Free Software Foundation, 
	Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307 USA

	Program written and compiled with Borland C++ Builder 5
	Programming and design: 	Alan Lenton
	Home website:					www.ibgames.net
-----------------------------------------------------------------------*/

#include <vcl.h>
#pragma hdrstop

#include "new_account.h"

#include <cctype>
#include <cstdlib>
#include <cstring>

#include "coms.h"
#include "fed2.h"
#include "new_ac_code.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TNewAccountForm *NewAccountForm;
//---------------------------------------------------------------------------

__fastcall TNewAccountForm::TNewAccountForm(TComponent* Owner)
	: TForm(Owner)
{
	new_account =0;
	can_close = true;
}


void __fastcall TNewAccountForm::CancelBtnClick(TObject *Sender)
{
	can_close = true;
}

bool	TNewAccountForm::CheckEMail()
{
	if(!NewAcCheck->Checked)
		return(true);
	if(EMailEdit->Text.Pos("@") ==0)
	{
		MessageDlg("Invalid e-mail address",mtError,TMsgDlgButtons() << mbOK,0);
		return(false);
	}
	else
		return(true);
}

bool	TNewAccountForm::CheckForASCII(const std::string& text) const
{
	int len = text.length();
	if(std::isalpha(text[0]) == 0)
	{
		MessageDlg("Name and species must start with a letter.",mtError,TMsgDlgButtons() << mbOK,0);
		return(false);
	}
	for(int count = 0;count < len;count++)
	{
		if((std::isalnum(text[count]) == 0) || (text[count] == ' '))
		{
			MessageDlg("Letters and numbers only in name and species",mtError,TMsgDlgButtons() << mbOK,0);
			return(false);
		}
	}
	return(true);
}

bool	TNewAccountForm::CheckHost()
{
	if(HostEdit->Text.Length() == 0)
	{
		MessageDlg("No host server name given",mtError,TMsgDlgButtons() << mbOK,0);
		return(false);
	}
	else
		return(true);
}

bool	TNewAccountForm::CheckID()
{
	if(IDEdit->Text.Length() < 5)
	{
		MessageDlg("Account IDs must be at least 5 letters",mtError,TMsgDlgButtons() << mbOK,0);
		return(false);
	}
	else
	{
		std::string	text(NameEdit->Text.c_str());
		return(CheckForASCII(text));
 	}
}

bool	TNewAccountForm::CheckName()
{
	if(NameEdit->Text.Length() < 3)
	{
		MessageDlg("Names must be at least 3 letters",mtError,TMsgDlgButtons() << mbOK,0);
		return(false);
	}
	else
	{
		std::string	text(NameEdit->Text.c_str());
		return(CheckForASCII(text));
	}
}

bool	TNewAccountForm::CheckPassword()
{
	if(PwEdit->Text.Length() < 8)
	{
		MessageDlg("Passwordss must be at least 8 letters",mtError,TMsgDlgButtons() << mbOK,0);
		return(false);
	}
	else
		return(ConfirmPassword());
}

bool	TNewAccountForm::CheckPort()
{
	if(PortEdit->Text.Length() == 0)
	{
		MessageDlg("No host port number given",mtError,TMsgDlgButtons() << mbOK,0);
		return(false);
	}
	else
		return(true);
}

bool	TNewAccountForm::CheckRace()
{
	if(RaceEdit->Text.Length() < 3)
	{
		MessageDlg("Species names must be at least 3 letters",mtError,TMsgDlgButtons() << mbOK,0);
		return(false);
	}
	else
	{
		std::string	text(RaceEdit->Text.c_str());
		return(CheckForASCII(text));
	}
}

bool	TNewAccountForm::CheckUnused()
{
	if(std::atoi(UnusedEdit->Text.c_str()) == 0)
		return(true);
	if(MessageDlg("You have not used all your stat points\nContinue anyway?",
										mtError,TMsgDlgButtons() << mbYes << mbNo,0) == mrYes)
		return(true);
	else
		return(false);
}

bool	TNewAccountForm::ConfirmPassword()
{
	if(!NewAcCheck->Checked)
		return(true);
	if(PwEdit->Text != PwConfEdit->Text)
	{
		MessageDlg("Your password does not match the confirmation",mtError,TMsgDlgButtons() << mbOK,0);
		PwEdit->Text = "";
		PwConfEdit->Text = "";
		return(false);
	}
	return(true);
}

void __fastcall TNewAccountForm::FormCloseQuery(TObject *Sender,
		bool &CanClose)
{
	CanClose = can_close;
}

void __fastcall TNewAccountForm::SendBtnClick(TObject *Sender)
{
	if(!CheckHost() || !CheckPort() ||
				!CheckID() || !CheckPassword() || !CheckEMail() ||
						!CheckName() || !CheckRace() || !CheckUnused())
		can_close = false;
	else
	{
		can_close = true;
		new_account->Connect();
	}
}

void __fastcall TNewAccountForm::StatChange(TObject *Sender,
		bool &AllowChange, short NewValue, TUpDownDirection Direction)
{
	int stats_left = std::atoi(UnusedEdit->Text.c_str());
	if(Direction == updNone)
	{
		AllowChange = false;
		return;
	}

	if(Direction == updUp)
	{
		if(stats_left > 0)
		{
			UnusedEdit->Text = --stats_left;
			AllowChange = true;
		}
		else
		{
			MessageDlg("You have no stats left to allocate!",mtError,TMsgDlgButtons() << mbOK,0);
			AllowChange = false;
		}
	}
	else
	{
		UnusedEdit->Text = ++stats_left;
		AllowChange = true;
	}
}


